"""Define sensor models."""

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import Any, Literal

import ciso8601
from mashumaro import DataClassDictMixin, field_options

from aionotion.const import LOGGER


@dataclass(frozen=True, kw_only=True)
class ListenerLocalizedStatus(DataClassDictMixin):
    """Define a localized listener status."""

    state: str
    description: str


@dataclass(frozen=True, kw_only=True)
class InsightOrigin(DataClassDictMixin):
    """Define an insight origin."""

    id: str | None = None
    type: str | None = None


@dataclass(frozen=True, kw_only=True)
class PrimaryListenerInsight(DataClassDictMixin):
    """Define a primary listener insight."""

    origin: InsightOrigin | None
    value: str | None
    data_received_at: datetime | None = field(
        default=None, metadata={"deserialize": ciso8601.parse_datetime}
    )


@dataclass(frozen=True, kw_only=True)
class ListenerInsights(DataClassDictMixin):
    """Define listener insights:"""

    primary: PrimaryListenerInsight


class ListenerKind(Enum):
    """Define the kinds of listener."""

    BATTERY = 0
    WATER_FLOW = 1
    MOLD = 2
    TEMPERATURE = 3
    LEAK = 4
    SAFE = 5
    DOOR = 6
    ALARM = 7
    SENSOR_CONNECTION = 10
    WINDOW_HINGED_VERTICAL = 12
    GARAGE_DOOR = 13
    WINDOW_HINGED_HORIZONTAL = 16
    SYSTEM_OCCPUANCY = 23
    SENSOR_FIRMWARE = 24
    BRIDGE_FIRMWARE = 25
    BRIDGE_CONNECTION = 26
    SLIDING_DOOR_OR_WINDOW = 32
    SYSTEM_USER_OCCUPANCY = 33
    ESCALATION = 34
    SYSTEM_STATUS = 35
    UNKNOWN = 99


@dataclass(frozen=True, kw_only=True)
class Listener(DataClassDictMixin):
    """Define a listener."""

    id: str
    definition_id: int
    created_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})
    model_version: str
    sensor_id: str
    status_localized: ListenerLocalizedStatus
    insights: ListenerInsights
    configuration: dict[str, Any]
    pro_monitoring_status: Literal["eligible", "ineligible"]
    device_type: str = field(metadata=field_options(alias="type"))
    kind: ListenerKind = field(init=False)

    def __post_init__(self) -> None:
        """Perform post-init initialization."""
        try:
            object.__setattr__(self, "kind", ListenerKind(self.definition_id))
        except ValueError:
            LOGGER.info("Unknown listener kind: %s", self.definition_id)
            object.__setattr__(self, "kind", ListenerKind.UNKNOWN)


@dataclass(frozen=True, kw_only=True)
class ListenerAllResponse(DataClassDictMixin):
    """Define an API response containing all listeners."""

    listeners: list[Listener]


@dataclass(frozen=True, kw_only=True)
class ListenerDefinition(DataClassDictMixin):
    """Define an API response containing all listener definitions."""

    id: int
    name: str
    conflict_type: str
    priority: int
    hidden: bool
    conflicting_types: list[str]
    resources: dict | None
    compatible_hardware_revisions: list[int]
    type: str


@dataclass(frozen=True, kw_only=True)
class ListenerDefinitionResponse(DataClassDictMixin):
    """Define an API response containing all listener definitions."""

    listener_definitions: list[ListenerDefinition]



"""Define datetime utilities."""

from datetime import datetime

try:
    from datetime import UTC
except ImportError:
    # In place for support of Python 3.10
    from datetime import timezone

    UTC = timezone.utc


def utcnow() -> datetime:
    """Return the current UTC time.

    Returns:
        A ``datetime.datetime`` object.
    """
    return datetime.now(tz=UTC)


def utc_from_timestamp(timestamp: float) -> datetime:
    """Return a UTC time from a timestamp.

    Args:
        timestamp: The epoch to convert.

    Returns:
        A parsed ``datetime.datetime`` object.
    """
    return datetime.fromtimestamp(timestamp, tz=UTC)



"""Define auth utilities."""

from typing import Any

import jwt


def decode_jwt(encoded_jwt: str) -> dict[str, Any]:
    """Decode and return a JWT.

    Args:
        encoded_jwt: An encoded JWT.

    Returns:
        A decoded JWT.
    """
    return jwt.decode(
        encoded_jwt,
        "secret",
        algorithms=["HS256"],
        options={"verify_signature": False},
    )



"""Define system models."""

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime

import ciso8601
from mashumaro import DataClassDictMixin


@dataclass(frozen=True, kw_only=True)
class System(DataClassDictMixin):
    """Define a system."""

    uuid: str
    name: str
    mode: str
    partners: list[str]
    latitude: float
    longitude: float
    timezone_id: str
    created_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})
    updated_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})
    night_time_start: datetime = field(
        metadata={"deserialize": ciso8601.parse_datetime}
    )
    night_time_end: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})
    id: int
    locality: str
    postal_code: str
    administrative_area: str
    fire_number: str
    police_number: str
    emergency_number: str
    address: str | None
    notion_pro_permit: str | None


@dataclass(frozen=True, kw_only=True)
class SystemAllResponse(DataClassDictMixin):
    """Define an API response containing all systems."""

    systems: list[System]


@dataclass(frozen=True, kw_only=True)
class SystemGetResponse(DataClassDictMixin):
    """Define an API response containing a single system."""

    systems: System



"""Define a base client for interacting with Notion."""

from __future__ import annotations

import asyncio
from collections.abc import Callable
from datetime import datetime
from typing import Any, TypeVar, cast
from uuid import uuid4

from aiohttp import ClientSession, ClientTimeout
from aiohttp.client_exceptions import ClientError
from mashumaro import DataClassDictMixin
from mashumaro.exceptions import (
    MissingField,
    SuitableVariantNotFoundError,
    UnserializableDataError,
)

from aionotion.bridge import Bridge
from aionotion.const import LOGGER
from aionotion.errors import InvalidCredentialsError, RequestError
from aionotion.listener import Listener
from aionotion.sensor import Sensor
from aionotion.system import System
from aionotion.user import User
from aionotion.user.models import (
    AuthenticateViaCredentialsLegacyResponse,
    AuthenticateViaCredentialsResponse,
    AuthenticateViaRefreshTokenResponse,
)
from aionotion.util.auth import decode_jwt
from aionotion.util.dt import utc_from_timestamp, utcnow

API_BASE = "https://api.getnotion.com/api"

DEFAULT_TIMEOUT = 10

NotionBaseModelT = TypeVar("NotionBaseModelT", bound=DataClassDictMixin)
RefreshTokenCallbackT = Callable[[str], None]


def get_token_header_value(access_token: str, refresh_token: str | None) -> str:
    """Return the value for the Authorization header.

    The old API uses a different format for the Authorization header than the new API.
    We detect whether we're using the new API by checking whether a refresh token is
    present.

    Args:
        access_token: An access token.
        refresh_token: A refresh token (if it exists).

    Returns:
        The value for the Authorization header.
    """
    if refresh_token:
        return f"Bearer {access_token}"
    return f"Token token={access_token}"


class Client:
    """Define the API object."""

    def __init__(
        self, *, session: ClientSession | None = None, session_name: str | None = None
    ) -> None:
        """Initialize.

        Args:
            session: An optional aiohttp ClientSession.
            session_name: An optional session name to use for authentication.
        """
        self._access_token: str | None = None
        self._access_token_expires_at: datetime | None = None
        self._refresh_event = asyncio.Event()
        self._refresh_lock = asyncio.Lock()
        self._refresh_token: str | None = None
        self._refresh_token_callbacks: list[RefreshTokenCallbackT] = []
        self._refreshing = False
        self._session = session
        self._session_name = session_name or uuid4().hex
        self.user_uuid: str = ""

        self.bridge = Bridge(self)
        self.listener = Listener(self)
        self.sensor = Sensor(self)
        self.system = System(self)
        self.user = User(self)

    @property
    def refresh_token(self) -> str | None:
        """Return the refresh token."""
        return self._refresh_token

    def _save_tokens_from_auth_response(
        self,
        auth_response: AuthenticateViaCredentialsResponse
        | AuthenticateViaRefreshTokenResponse,
    ) -> None:
        """Save the authentication and refresh tokens from an auth response.

        Args:
            auth: An API response containing auth info.
        """
        self._access_token = auth_response.auth.jwt
        self._refresh_token = auth_response.auth.refresh_token

        # Determine the expiration time of the access token:
        decoded_jwt = decode_jwt(self._access_token)
        self._access_token_expires_at = utc_from_timestamp(decoded_jwt["exp"])

        # Call all refresh token callbacks:
        for callback in self._refresh_token_callbacks:
            callback(self._refresh_token)

    def add_refresh_token_callback(
        self, callback: RefreshTokenCallbackT
    ) -> Callable[[], None]:
        """Add a callback to be called when the refresh token is updated."""
        self._refresh_token_callbacks.append(callback)

        def remove_callback() -> None:
            """Remove the callback from the list of callbacks."""
            self._refresh_token_callbacks.remove(callback)

        return remove_callback

    async def async_authenticate_from_credentials(
        self, email: str, password: str
    ) -> None:
        """Authenticate via username and password.

        Args:
            email: The email address of a Notion account.
            password: The account password.
        """
        auth_response: AuthenticateViaCredentialsResponse = (
            await self.async_request_and_validate(
                "post",
                "/auth/login",
                AuthenticateViaCredentialsResponse,
                headers={"Accept-Version": "2"},
                json={
                    "auth": {
                        "email": email,
                        "password": password,
                        "session_name": self._session_name,
                    }
                },
            )
        )

        self.user_uuid = auth_response.user.uuid
        self._save_tokens_from_auth_response(auth_response)

    async def async_authenticate_from_refresh_token(
        self, *, refresh_token: str | None = None
    ) -> None:
        """Authenticate via a refresh token.

        Args:
            refresh_token: The refresh token to use. If not provided, the refresh token
                that was used to authenticate the user initially will be used.

        Raises:
            InvalidCredentialsError: If no refresh token is provided and the user has
                not been authenticated yet.
        """
        if refresh_token is None and self._refresh_token is None:
            raise InvalidCredentialsError("No valid refresh token provided")

        self._refreshing = True

        async with self._refresh_lock:
            # If a refresh token is explicitly provided, use it:
            if refresh_token:
                self._refresh_token = refresh_token

            assert self._refresh_token is not None

            self._refresh_event.clear()

            try:
                auth_response: AuthenticateViaRefreshTokenResponse = (
                    await self.async_request_and_validate(
                        "post",
                        f"/auth/{self.user_uuid}/refresh",
                        AuthenticateViaRefreshTokenResponse,
                        refresh_request=True,
                        headers={"Accept-Version": "2"},
                        json={
                            "auth": {
                                "refresh_token": self._refresh_token,
                            }
                        },
                    )
                )
                self._save_tokens_from_auth_response(auth_response)
            finally:
                self._refreshing = False
                self._refresh_event.set()

    async def async_legacy_authenticate_from_credentials(
        self, email: str, password: str
    ) -> None:
        """Authenticate via username and password (via a legacy endpoint).

        This is kept in place for compatibility, but should be considered deprecated.

        Args:
            email: The email address of a Notion account.
            password: The account password.
        """
        LOGGER.warning(
            "Using legacy authentication endpoint; this is deprecated and will be "
            "removed in a future release"
        )
        auth_response: AuthenticateViaCredentialsLegacyResponse = (
            await self.async_request_and_validate(
                "post",
                "/users/sign_in",
                AuthenticateViaCredentialsLegacyResponse,
                json={
                    "sessions": {
                        "email": email,
                        "password": password,
                    }
                },
            )
        )

        self.user_uuid = auth_response.users.uuid
        self._access_token = auth_response.session.authentication_token

    async def async_request(
        self,
        method: str,
        endpoint: str,
        *,
        refresh_request: bool = False,
        **kwargs: dict[str, Any],
    ) -> dict[str, Any]:
        """Make an API request.

        Args:
            method: An HTTP method.
            endpoint: A relative API endpoint.
            refresh_request: Whether this is a request to refresh the access token.
            **kwargs: Additional kwargs to send with the request.

        Returns:
            An API response payload.

        Raises:
            InvalidCredentialsError: Raised upon invalid credentials.
            RequestError: Raised upon an underlying HTTP error.
        """
        if self._access_token_expires_at and utcnow() >= self._access_token_expires_at:
            LOGGER.debug("Access token expired, refreshing...")
            self._access_token = None
            self._access_token_expires_at = None
            await self.async_authenticate_from_refresh_token()

        # If an authenticated request arrives while we're refreshing, hold until the
        # refresh process is done:
        if not refresh_request and self._refreshing:
            await self._refresh_event.wait()

        url: str = f"{API_BASE}{endpoint}"

        kwargs.setdefault("headers", {})
        if self._access_token:
            kwargs["headers"]["Authorization"] = get_token_header_value(
                self._access_token, self._refresh_token
            )

        if use_running_session := self._session and not self._session.closed:
            session = self._session
        else:
            session = ClientSession(timeout=ClientTimeout(total=DEFAULT_TIMEOUT))

        data: dict[str, Any] = {}

        async with session.request(method, url, **kwargs) as resp:
            data = await resp.json()

            try:
                resp.raise_for_status()
            except ClientError as err:
                if "401" in str(err):
                    raise InvalidCredentialsError("Invalid credentials") from err
                raise RequestError(data["errors"][0]["title"]) from err

        if not use_running_session:
            await session.close()

        LOGGER.debug("Received data from %s: %s", endpoint, data)

        return data

    async def async_request_and_validate(
        self,
        method: str,
        endpoint: str,
        model: type[DataClassDictMixin],
        *,
        refresh_request: bool = False,
        **kwargs: dict[str, Any],
    ) -> NotionBaseModelT:
        """Make an API request and validate the response against a Pydantic model.

        Args:
            method: An HTTP method.
            endpoint: A relative API endpoint.
            model: A Pydantic model to validate the response against.
            refresh_request: Whether this is a request to refresh the access token.
            **kwargs: Additional kwargs to send with the request.

        Returns:
            A parsed, validated Pydantic model representing the response.
        """
        raw_data = await self.async_request(
            method, endpoint, refresh_request=refresh_request, **kwargs
        )

        try:
            return cast(NotionBaseModelT, model.from_dict(raw_data))
        except (
            MissingField,
            SuitableVariantNotFoundError,
            UnserializableDataError,
        ) as err:
            raise RequestError(
                f"Error while parsing response from {endpoint}: {err}"
            ) from err


async def async_get_client_with_credentials(
    email: str,
    password: str,
    *,
    session: ClientSession | None = None,
    session_name: str | None = None,
    use_legacy_auth: bool = False,
) -> Client:
    """Return an authenticated API object (using username/password)

    Args:
        email: The email address of a Notion account.
        password: The account password.
        session: An optional aiohttp ClientSession.
        session_name: An optional session name to use for authentication.
        use_legacy_auth: Whether to use the legacy authentication endpoint.

    Returns:
        An authenticated Client object.
    """
    client = Client(session=session, session_name=session_name)
    if use_legacy_auth:
        await client.async_legacy_authenticate_from_credentials(email, password)
    else:
        await client.async_authenticate_from_credentials(email, password)
    return client


# Alias for backwards compatibility:
async_get_client = async_get_client_with_credentials


async def async_get_client_with_refresh_token(
    user_uuid: str,
    refresh_token: str,
    *,
    session: ClientSession | None = None,
    session_name: str | None = None,
) -> Client:
    """Return an authenticated API object (using a refresh token).

    Args:
        user_uuid: The UUID of the user.
        refresh_token: A refresh token.
        session: An optional aiohttp ClientSession.
        session_name: An optional session name to use for authentication.

    Returns:
        An authenticated Client object.
    """
    client = Client(session=session, session_name=session_name)
    client.user_uuid = user_uuid
    await client.async_authenticate_from_refresh_token(refresh_token=refresh_token)
    return client



"""Define the aionotion package."""

from .client import async_get_client  # noqa: F401
from .client import async_get_client_with_credentials  # noqa: F401
from .client import async_get_client_with_refresh_token  # noqa: F401



"""Define user models."""

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime

import ciso8601
from mashumaro import DataClassDictMixin


@dataclass(frozen=True, kw_only=True)
class AuthTokens(DataClassDictMixin):
    """Define auth tokens."""

    jwt: str
    refresh_token: str


@dataclass(frozen=True, kw_only=True)
class LegacySession(DataClassDictMixin):
    """Define a legacy Notion session."""

    user_id: str
    authentication_token: str


@dataclass(frozen=True, kw_only=True)
class LegacyUser(DataClassDictMixin):
    """Define a legacy Notion user."""

    id: int
    uuid: str
    first_name: str
    last_name: str
    email: str
    phone_number: str | None
    role: str
    organization: str
    authentication_token: str
    created_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})
    updated_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})


@dataclass(frozen=True, kw_only=True)
class User(DataClassDictMixin):
    """Define a Notion user."""

    id: int
    uuid: str
    first_name: str
    last_name: str
    email: str
    phone_number: str | None
    role: str
    organization: str
    created_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})
    updated_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})


@dataclass(frozen=True, kw_only=True)
class UserInformationResponse(DataClassDictMixin):
    """Define an API response containing user information."""

    users: User


@dataclass(frozen=True, kw_only=True)
class AuthenticateViaCredentialsResponse(DataClassDictMixin):
    """Define an API response for authentication via credentials."""

    user: User
    auth: AuthTokens


@dataclass(frozen=True, kw_only=True)
class AuthenticateViaCredentialsLegacyResponse(DataClassDictMixin):
    """Define an API response for authentication via credentials (legacy)."""

    users: LegacyUser
    session: LegacySession


@dataclass(frozen=True, kw_only=True)
class AuthenticateViaRefreshTokenResponse(DataClassDictMixin):
    """Define an API response for authentication via refresh token."""

    auth: AuthTokens


@dataclass(frozen=True, kw_only=True)
class UserPreferences(DataClassDictMixin):
    """Define user preferences."""

    user_id: int
    military_time_enabled: bool
    celsius_enabled: bool
    disconnect_alerts_enabled: bool
    home_away_alerts_enabled: bool
    battery_alerts_enabled: bool


@dataclass(frozen=True, kw_only=True)
class UserPreferencesResponse(DataClassDictMixin):
    """Define an API response containing all devices."""

    user_preferences: UserPreferences



"""Define sensor models."""

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime

import ciso8601
from mashumaro import DataClassDictMixin


@dataclass(frozen=True, kw_only=True)
class Bridge(DataClassDictMixin):
    """Define a bridge representation."""

    id: int
    hardware_id: str


@dataclass(frozen=True, kw_only=True)
class Firmware(DataClassDictMixin):
    """Define firmware information."""

    status: str


@dataclass(frozen=True, kw_only=True)
class SurfaceType(DataClassDictMixin):
    """Define a surface type."""

    id: str
    name: str
    slug: str


@dataclass(frozen=True, kw_only=True)
class User(DataClassDictMixin):
    """Define a user representation."""

    id: int
    email: str


@dataclass(frozen=True, kw_only=True)
class Sensor(DataClassDictMixin):  # pylint: disable=too-many-instance-attributes
    """Define a sensor."""

    id: int
    uuid: str
    user: User
    bridge: Bridge
    last_bridge_hardware_id: str
    name: str
    location_id: int
    system_id: int
    hardware_id: str
    hardware_revision: int
    firmware_version: str
    device_key: str
    encryption_key: bool
    installed_at: datetime | None = field(
        default=None, metadata={"deserialize": ciso8601.parse_datetime}
    )
    calibrated_at: datetime | None = field(
        default=None, metadata={"deserialize": ciso8601.parse_datetime}
    )
    last_reported_at: datetime | None = field(
        default=None, metadata={"deserialize": ciso8601.parse_datetime}
    )
    missing_at: datetime | None = field(
        default=None, metadata={"deserialize": ciso8601.parse_datetime}
    )
    updated_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})
    created_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})
    signal_strength: int
    firmware: Firmware
    surface_type: SurfaceType | None


@dataclass(frozen=True, kw_only=True)
class SensorAllResponse(DataClassDictMixin):
    """Define an API response containing all sensors."""

    sensors: list[Sensor]


@dataclass(frozen=True, kw_only=True)
class SensorGetResponse(DataClassDictMixin):
    """Define an API response containing a single sensor."""

    sensors: Sensor



"""Define package errors."""


class NotionError(Exception):
    """Define a base error."""

    pass


class RequestError(NotionError):
    """Define an error related to invalid requests."""

    pass


class InvalidCredentialsError(NotionError):
    """Define an error for unauthenticated accounts."""

    pass



"""Define endpoints for interacting with bridges."""

from __future__ import annotations

from typing import TYPE_CHECKING

from aionotion.bridge.models import Bridge as BridgeModel
from aionotion.bridge.models import BridgeAllResponse, BridgeGetResponse

if TYPE_CHECKING:
    from aionotion.client import Client


class Bridge:
    """Define an object to interact with bridge endpoints."""

    def __init__(self, client: Client) -> None:
        """Initialize.

        Args:
            client: The aionotion client
        """
        self._client = client

    async def async_all(self) -> list[BridgeModel]:
        """Get all bridges.

        Returns:
            A validated API response payload.
        """
        response: BridgeAllResponse = await self._client.async_request_and_validate(
            "get", "/base_stations", BridgeAllResponse
        )
        return response.base_stations

    async def async_get(self, bridge_id: int) -> BridgeModel:
        """Get a bridge by ID.

        Args:
            bridge_id: The ID of the bridge to get.

        Returns:
            A validated API response payload.
        """
        response: BridgeGetResponse = await self._client.async_request_and_validate(
            "get", f"/base_stations/{bridge_id}", BridgeGetResponse
        )
        return response.base_stations



"""Define bridge models."""

from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime

import ciso8601
from mashumaro import DataClassDictMixin


@dataclass(frozen=True, kw_only=True)
class FirmwareVersion(DataClassDictMixin):
    """Define firmware version info."""

    wifi: str
    wifi_app: str
    silabs: str | None = None
    ti: str | None = None


@dataclass(frozen=True, kw_only=True)
class Bridge(DataClassDictMixin):
    """Define a bridge."""

    id: int
    name: str | None
    mode: str
    hardware_id: str
    hardware_revision: int
    firmware_version: FirmwareVersion
    missing_at: datetime | None = field(
        default=None, metadata={"deserialize": ciso8601.parse_datetime}
    )
    created_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})
    updated_at: datetime = field(metadata={"deserialize": ciso8601.parse_datetime})
    system_id: int
    firmware: FirmwareVersion
    links: dict[str, int | str]


@dataclass(frozen=True, kw_only=True)
class BridgeAllResponse(DataClassDictMixin):
    """Define an API response containing all bridges."""

    base_stations: list[Bridge]


@dataclass(frozen=True, kw_only=True)
class BridgeGetResponse(DataClassDictMixin):
    """Define an API response containing a single bridge."""

    base_stations: Bridge



"""Define endpoints for interacting with systems (accounts)."""

from __future__ import annotations

from typing import TYPE_CHECKING

from aionotion.system.models import System as SystemModel
from aionotion.system.models import SystemAllResponse, SystemGetResponse

if TYPE_CHECKING:
    from aionotion.client import Client


class System:
    """Define an object to interact with system endpoints."""

    def __init__(self, client: Client) -> None:
        """Initialize.

        Args:
            client: The aionotion client
        """
        self._client = client

    async def async_all(self) -> list[SystemModel]:
        """Get all systems.

        Returns:
            An API response payload.
        """
        response: SystemAllResponse = await self._client.async_request_and_validate(
            "get", "/systems", SystemAllResponse
        )
        return response.systems

    async def async_get(self, system_id: int) -> SystemModel:
        """Get a system by ID.

        Args:
            system_id: The ID of the system to get.

        Returns:
            An API response payload.
        """
        response: SystemGetResponse = await self._client.async_request_and_validate(
            "get", f"/systems/{system_id}", SystemGetResponse
        )
        return response.systems



"""Define endpoints for interacting with listeners."""

from __future__ import annotations

from typing import TYPE_CHECKING

from aionotion.listener.models import Listener as ListenerModel
from aionotion.listener.models import (
    ListenerAllResponse,
    ListenerDefinition,
    ListenerDefinitionResponse,
)

if TYPE_CHECKING:
    from aionotion.client import Client


class Listener:
    """Define an object to interact with sensor endpoints."""

    def __init__(self, client: Client) -> None:
        """Initialize.

        Args:
            client: The aionotion client
        """
        self._client = client

    async def async_all(self) -> list[ListenerModel]:
        """Get all listeners.

        Returns:
            A validated API response payload.
        """
        response: ListenerAllResponse = await self._client.async_request_and_validate(
            "get", "/sensor/listeners", ListenerAllResponse
        )
        return response.listeners

    async def async_definitions(self) -> list[ListenerDefinition]:
        """Get all listener definitions.

        Returns:
            A validated API response payload.
        """
        response: ListenerDefinitionResponse = (
            await self._client.async_request_and_validate(
                "get", "/listener_definitions", ListenerDefinitionResponse
            )
        )
        return response.listener_definitions



"""Define endpoints for interacting with users."""

from __future__ import annotations

from typing import TYPE_CHECKING

from aionotion.user.models import User as UserModel
from aionotion.user.models import (
    UserInformationResponse,
    UserPreferences,
    UserPreferencesResponse,
)

if TYPE_CHECKING:
    from aionotion.client import Client


class User:  # pylint: disable=too-few-public-methods
    """Define an object to interact with user endpoints."""

    def __init__(self, client: Client) -> None:
        """Initialize.

        Args:
            client: The aionotion client
        """
        self._client = client

    async def async_info(self) -> UserModel:
        """Get the user's information.

        Returns:
            A validated API response payload.
        """
        response: UserInformationResponse = (
            await self._client.async_request_and_validate(
                "get",
                f"/users/{self._client.user_uuid}",
                UserInformationResponse,
            )
        )
        return response.users

    async def async_preferences(self) -> UserPreferences:
        """Get user preferences.

        Returns:
            A validated API response payload.
        """
        response: UserPreferencesResponse = (
            await self._client.async_request_and_validate(
                "get",
                f"/users/{self._client.user_uuid}/user_preferences",
                UserPreferencesResponse,
            )
        )
        return response.user_preferences



"""Define endpoints for interacting with sensors."""

from __future__ import annotations

from typing import TYPE_CHECKING

from aionotion.sensor.models import Sensor as SensorModel
from aionotion.sensor.models import SensorAllResponse, SensorGetResponse

if TYPE_CHECKING:
    from aionotion.client import Client


class Sensor:
    """Define an object to interact with sensor endpoints."""

    def __init__(self, client: Client) -> None:
        """Initialize.

        Args:
            client: The aionotion client
        """
        self._client = client

    async def async_all(self) -> list[SensorModel]:
        """Get all sensors.

        Returns:
            A validated API response payload.
        """
        response: SensorAllResponse = await self._client.async_request_and_validate(
            "get", "/sensors", SensorAllResponse
        )
        return response.sensors

    async def async_get(self, sensor_id: int) -> SensorModel:
        """Get a sensor by ID.

        Args:
            sensor_id: The ID of the sensor to get.

        Returns:
            A validated API response payload.
        """
        response: SensorGetResponse = await self._client.async_request_and_validate(
            "get", f"/sensors/{sensor_id}", SensorGetResponse
        )
        return response.sensors
